# Class: pe_puppetdb::pe::database
#
# This class manages a PE postgresql server and database instance suitable for use
# with PE puppetdb.  It uses the `puppetlabs/puppetlabs-pe_postgresql` puppet module for getting
# the postgres server up and running, and then also for creating the pe-puppetdb
# database instance and user account.
#
# Parameters:
#   ['listen_addresses']     - A comma-separated list of hostnames or IP addresses
#                              on which the postgres server should listen for incoming
#                              connections.  (Defaults to 'localhost'.  This parameter
#                              maps directly to postgresql's 'listen_addresses' config
#                              option; use a '*' to allow connections on any accessible
#                              address.
#   ['database_name']        - The name of the database instance to connect to.
#                              (defaults to `pe-puppetdb`)
#   ['database_username']    - The name of the database user to connect as.
#                              (defaults to `pe-puppetdb`)
#   ['database_password']    - The password for the database user.
#                              (defaults to `pe-puppetdb`)
#   ['tablespace_name']      - The name of puppetdb tablespace.
#   ['tablespace_location']  - The location of puppetdb tablespace.
# Actions:
# - Creates and manages a postgres server and database instance for use by
#   puppetdb
#
# Requires:
# - `puppetlabs/puppetlabs-pe_postgresql`
# - `puppetlabs/puppetlabs-stdlib`
#
# Sample Usage:
#   class { 'pe_puppetdb::pe::database':
#       listen_addresses         => 'my.postgres.host.name',
#   }
#
class pe_puppetdb::pe::database(
  # TODO: expose more of the parameters from `inkling/postgresql`!
  $listen_addresses       = $pe_puppetdb::pe::params::database_host,
  $database_name          = $pe_puppetdb::pe::params::database_name,
  $database_username      = $pe_puppetdb::pe::params::database_username,
  $database_password      = $pe_puppetdb::pe::params::database_password,
  $tablespace_name        = $pe_puppetdb::pe::params::tablespace_name,
  $tablespace_location    = $pe_puppetdb::pe::params::tablespace_location,
) inherits pe_puppetdb::pe::params {

  # get the pg server up and running
  class { 'pe_postgresql::pe':
    config_hash => {
      # TODO: make this stuff configurable
      'ip_mask_allow_all_users' => '0.0.0.0/0',
      'listen_addresses'        => $listen_addresses,
      'manage_redhat_firewall'  => false,
    },
  }

  # PostgreSQL configuration
  $database_config_hash = pe_puppetdb_safe_hiera_hash('pe_puppetdb::pe::database::database_config_hash', $pe_puppetdb::pe::params::default_database_settings)

  # 'memorysize' fact in bytes (can be used in database_config_hash)
  $memorysize_in_bytes = to_bytes($memorysize)

  # reserved memory for PuppetDB (can be used in database_config_hash)
  # '0 + x' is used for conversion to integer
  $reserved_non_postgresql_memory_in_bytes = 0 + pe_puppetdb_safe_hiera('pe_puppetdb::pe::database::reserved_non_postgresql_memory_in_bytes', $pe_puppetdb::pe::params::default_reserved_non_postgresql_memory_in_bytes)

  include "pe_postgresql::params"

  $pg_conf_include_file = "${pe_postgresql::params::confdir}/postgresql_puppet_extras.conf"

  # set kernel.shmmax to 50% of total RAM to be able to set shared_buffers to an appropriate value
  $shmmax = $memorysize_in_bytes / 2

  exec {'set kernel.shmmax':
    command => "sysctl -w kernel.shmmax=${shmmax}",
    path    => "/sbin/:/bin/",
    unless  => "sysctl kernel.shmmax | grep \"kernel.shmmax\s*=\s*${shmmax}$\""
  }

  ini_setting {'kernel.shmmax':
    ensure  => present,
    section => '',
    path    => '/etc/sysctl.conf',
    setting => 'kernel.shmmax',
    value   => $shmmax,
  }

  file { $pg_conf_include_file:
    content => template('pe_puppetdb/pe/hash_to_conf.erb'),
    notify  => Service['postgresqld'],
    require  => [Class['::pe_postgresql::initdb'], Exec['set kernel.shmmax']],
  }

  # create tablespace for puppetdb database
  pe_postgresql::tablespace { $tablespace_name:
    location => $tablespace_location,
    require  => [Class['::pe_postgresql::server'], Service['postgresqld']],
  }

  # create the puppetdb database
  pe_postgresql::db { $database_name:
    tablespace => $tablespace_name,
    user       => $database_username,
    password   => $database_password,
    grant      => 'all',
    require    => Pe_postgresql::Tablespace[$tablespace_name],
  }
}
