require 'spec_helper'
require 'puppetx/puppetlabs/pe_console/config'

describe Puppetx::Puppetlabs::PEConsole::Config do
  describe "#load" do
    let(:confdir) do
      temp = Tempfile.new('confdir')
      path = temp.path
      temp.close!
      Dir.mkdir(path)
      path
    end

    before :each do
      Puppet[:confdir] = confdir
    end

    after :each do
      FileUtils.rm_rf(confdir)
    end

    describe "with no config file" do
      it "should use the default settings" do
        config = described_class.load
        config.server.should == 'localhost'
        config.certificate_name.should == 'pe-internal-dashboard'
        config.port.should == 443
        config.report_path.should == '/reports/upload'
      end

    end

    describe "with a config file" do
      def write_config(content)
        conf = File.join(confdir, 'console.conf')
        File.open(conf, 'w') { |file| file.print(content) }
      end

      it "should not explode if there are comments in the config file" do
        write_config <<CONF
#this is a comment
 ; so is this
[main]
server = main_server
   # yet another comment
port = 1234
CONF
        expect { described_class.load }.to_not raise_error
      end


      it "should use the config value if specified" do
        write_config <<CONF
[main]
server = main_server
certificate_name = classifier_service
port = 1234
report_path = /reports
CONF
        config = described_class.load
        config.server.should == 'main_server'
        config.certificate_name.should == 'classifier_service'
        config.port.should == 1234
        config.report_path.should == '/reports'
      end

      it "should use the default if no value is specified" do
        write_config ''

        config = described_class.load
        config.server.should == 'localhost'
        config.certificate_name.should == 'pe-internal-dashboard'
        config.port.should == 443
        config.report_path == '/reports/upload'
      end

      it "should be insensitive to whitespace" do
        write_config <<CONF
[main]
  server = main_server
    port    =  1234
CONF
        config = described_class.load
        config.server.should == 'main_server'
        config.port.should == 1234
      end

      it "should accept valid hostnames" do
        write_config <<CONF
[main]
server = foo.example-thing.com
port = 8081
CONF

        config = described_class.load
        config.server.should == 'foo.example-thing.com'
        config.port.should == 8081
      end

      it "should raise if a setting is outside of a section" do
        write_config 'foo = bar'

        expect do
          described_class.load
        end.to raise_error(/Setting 'foo = bar' is illegal outside of section/)
      end

      it "should raise if an illegal line is encountered" do
        write_config 'foo bar baz'

        expect do
          described_class.load
        end.to raise_error(/Unparseable line 'foo bar baz'/)
      end

    end
  end
end
